@@ -3,7 +3,7 @@
  * area.
  **/
  
-function openNewTabWith(href, linkNode, event, securityCheck)
+function openNewTabWith(href, linkNode, event, securityCheck, postData)
 {
   if (securityCheck)
     urlSecurityCheck(href, document); 
@@ -33,7 +33,7 @@ function openNewTabWith(href, linkNode, event, securityCheck)
 
   // open link in new tab
   var browser = top.document.getElementById("content");  
-  var theTab = browser.addTab(href, getReferrer(document), originCharset);
+  var theTab = browser.addTab(href, getReferrer(document), originCharset, postData);
   if (!loadInBackground)
     browser.selectedTab = theTab;
   
@@ -41,7 +41,7 @@ function openNewTabWith(href, linkNode, event, securityCheck)
     markLinkVisited(href, linkNode);
 }
 
-function openNewWindowWith(href, linkNode, securityCheck) 
+function openNewWindowWith(href, linkNode, securityCheck, postData) 
 {
   if (securityCheck)
     urlSecurityCheck(href, document);
@@ -55,7 +55,7 @@ function openNewWindowWith(href, linkNode, securityCheck)
     charsetArg = "charset=" + window._content.document.characterSet;
 
   var referrer = getReferrer(document);
-  window.openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no", href, charsetArg, referrer);
+  window.openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no", href, charsetArg, referrer, postData);
   
   if (linkNode)
     markLinkVisited(href, linkNode);
@@ -63,22 +63,24 @@ function openNewWindowWith(href, linkNode, securityCheck)
 
 function markLinkVisited(href, linkNode)
 {
-   var globalHistory = Components.classes["@mozilla.org/browser/global-history;1"]
-                               .getService(Components.interfaces.nsIGlobalHistory);
-   if (!globalHistory.isVisited(href)) {
-     globalHistory.addPage(href);
-     var oldHref = linkNode.getAttribute("href");
-     if (typeof oldHref == "string") {
-       // Use setAttribute instead of direct assignment.
-       // (bug 217195, bug 187195)
-       linkNode.setAttribute("href", "");
-       linkNode.setAttribute("href", oldHref);
-     }
-     else {
-       // Converting to string implicitly would be a 
-       // minor security hole (similar to bug 202994).
-     }
-   }
+  var globalHistory = Components.classes["@mozilla.org/browser/global-history;2"]
+                                .getService(Components.interfaces.nsIGlobalHistory2);
+
+  var uri = makeURL(href);
+  if (!globalHistory.isVisited(uri)) {
+    globalHistory.addURI(uri, false, true);
+    var oldHref = linkNode.getAttribute("href");
+    if (typeof oldHref == "string") {
+      // Use setAttribute instead of direct assignment.
+      // (bug 217195, bug 187195)
+      linkNode.setAttribute("href", "");
+      linkNode.setAttribute("href", oldHref);
+    }
+    else {
+      // Converting to string implicitly would be a 
+      // minor security hole (similar to bug 202994).
+    }
+  }
 }
 
 function urlSecurityCheck(url, doc) 
@@ -86,11 +88,17 @@ function urlSecurityCheck(url, doc)
   // URL Loading Security Check
   var focusedWindow = doc.commandDispatcher.focusedWindow;
   var sourceURL = getContentFrameURI(focusedWindow);
+  var sourceURI = Components.classes["@mozilla.org/network/standard-url;1"]
+                            .createInstance(Components.interfaces.nsIURI);
+  sourceURI.spec = sourceURL;
+  var destURI = Components.classes["@mozilla.org/network/standard-url;1"]
+                          .createInstance(Components.interfaces.nsIURI);
+  destURI.spec = url;
   const nsIScriptSecurityManager = Components.interfaces.nsIScriptSecurityManager;
   var secMan = Components.classes["@mozilla.org/scriptsecuritymanager;1"]
                          .getService(nsIScriptSecurityManager);
   try {
-    secMan.checkLoadURIStr(sourceURL, url, nsIScriptSecurityManager.STANDARD);
+    secMan.checkLoadURI(sourceURI, destURI, nsIScriptSecurityManager.STANDARD);
   } catch (e) {
     throw "Load of " + url + " denied.";
   }
@@ -110,7 +118,10 @@ function isContentFrame(aFocusedWindow)
 function getContentFrameURI(aFocusedWindow)
 {
   var contentFrame = isContentFrame(aFocusedWindow) ? aFocusedWindow : window.content;
-  return Components.lookupMethod(contentFrame, 'location').call(contentFrame).href;
+  if (contentFrame)
+    return Components.lookupMethod(contentFrame, 'location').call(contentFrame).href;
+  else
+    return null;
 }
 
 function getReferrer(doc)
@@ -118,13 +129,13 @@ function getReferrer(doc)
   var focusedWindow = doc.commandDispatcher.focusedWindow;
   var sourceURL = getContentFrameURI(focusedWindow);
 
-  try {
-    var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);
-    uri.spec = sourceURL;
-    return uri;
-  } catch (e) {
-    return null;
+  if (sourceURL) {
+    try {
+      return makeURL(sourceURL);
+    }
+    catch (e) { }
   }
+  return null;
 }
 
 const kSaveAsType_Complete = 0;   // Save document with attached objects
@@ -136,7 +147,8 @@ const kSaveAsType_Text = 2;       // Save document, converting to plain text.
 //  - Context ->  Save Page/Frame As...
 //  - Context ->  Save Link As...
 //  - Context ->  Save Image As...
-//  - Alt-Click Save Link As
+//  - Alt-Click links in web pages
+//  - Alt-Click links in the UI
 //
 // Try saving each of these types:
 // - A complete webpage using File->Save Page As, and Context->Save Page As
@@ -149,9 +161,9 @@ const kSaveAsType_Text = 2;       // Save document, converting to plain text.
 // - A linked document using Save Link As...
 // - A linked document using Alt-click Save Link As...
 //
-function saveURL(aURL, aFileName, aFilePickerTitleKey, aShouldBypassCache, aSkipPrompt)
+function saveURL(aURL, aFileName, aFilePickerTitleKey, aShouldBypassCache, aSkipPrompt, aReferrer)
 {
-  saveInternal(aURL, null, aFileName, aFilePickerTitleKey, aShouldBypassCache, aSkipPrompt);
+  saveInternal(aURL, null, aFileName, aFilePickerTitleKey, aShouldBypassCache, aSkipPrompt, aReferrer);
 }
 
 function saveDocument(aDocument, aSkipPrompt)
@@ -166,7 +178,8 @@ function saveDocument(aDocument, aSkipPrompt)
 
 function saveInternal(aURL, aDocument, 
                       aFileName, aFilePickerTitleKey,
-                      aShouldBypassCache, aSkipPrompt)
+                      aShouldBypassCache, aSkipPrompt,
+                      aReferrer)
 {
   if (aSkipPrompt == undefined)
     aSkipPrompt = false;
@@ -177,7 +190,8 @@ function saveInternal(aURL, aDocument,
     filePickerTitle: aFilePickerTitleKey,
     document: aDocument,
     bypassCache: aShouldBypassCache,
-    window: window
+    window: window,
+    referrer: aReferrer
   };
   var sniffer = new nsHeaderSniffer(aURL, foundHeaderInfo, data, aSkipPrompt);
 }
@@ -221,6 +235,8 @@ function foundHeaderInfo(aSniffer, aData, aSkipPrompt)
   catch (e) {
     var saveAsTypeResult = { rv: 0 };
     file = getTargetFile(aData, aSniffer, contentType, isDocument, aSkipPrompt, saveAsTypeResult);
+    if (!file)
+      return;
     saveAsType = saveAsTypeResult.rv;
   }
 
@@ -231,7 +247,7 @@ function foundHeaderInfo(aSniffer, aData, aSkipPrompt)
   var persistArgs = {
     source      : source,
     contentType : (isDocument && saveAsType == kSaveAsType_Text) ? "text/plain" : contentType,
-    target      : file,
+    target      : makeFileURL(file),
     postData    : aData.document ? getPostData() : null,
     bypassCache : aData.bypassCache
   };
@@ -257,10 +273,7 @@ function foundHeaderInfo(aSniffer, aData, aSkipPrompt)
     var filesFolder = null;
     if (persistArgs.contentType != "text/plain") {
       // Create the local directory into which to save associated files. 
-      const nsILocalFile = Components.interfaces.nsILocalFile;
-      const lfContractID = "@mozilla.org/file/local;1";
-      filesFolder = Components.classes[lfContractID].createInstance(nsILocalFile);
-      filesFolder.initWithPath(persistArgs.target.path);
+      filesFolder = file.clone();
       
       var nameWithoutExtension = filesFolder.leafName;
       nameWithoutExtension = nameWithoutExtension.substring(0, nameWithoutExtension.lastIndexOf("."));
@@ -277,6 +290,9 @@ function foundHeaderInfo(aSniffer, aData, aSkipPrompt)
       encodingFlags |= nsIWBP.ENCODE_FLAGS_ABSOLUTE_LINKS;
       encodingFlags |= nsIWBP.ENCODE_FLAGS_NOFRAMES_CONTENT;        
     }
+    else {
+      encodingFlags |= nsIWBP.ENCODE_FLAGS_ENCODE_BASIC_ENTITIES;
+    }
     
     const kWrapColumn = 80;
     dl.init(aSniffer.uri, persistArgs.target, null, null, null, persist);
@@ -284,13 +300,14 @@ function foundHeaderInfo(aSniffer, aData, aSkipPrompt)
                          persistArgs.contentType, encodingFlags, kWrapColumn);
   } else {
     dl.init(source, persistArgs.target, null, null, null, persist);
-    persist.saveURI(source, null, null, persistArgs.postData, null, persistArgs.target);
+    var referrer = aData.referrer || getReferrer(document)
+    persist.saveURI(source, null, referrer, persistArgs.postData, null, persistArgs.target);
   }
 }
 
 function getTargetFile(aData, aSniffer, aContentType, aIsDocument, aSkipPrompt, aSaveAsTypeResult)
 {
-  aSaveAsTypeResult.rv = kSaveAsType_Complete;                                           
+  aSaveAsTypeResult.rv = kSaveAsType_Complete;
   
   // Determine what the 'default' string to display in the File Picker dialog 
   // should be. 
@@ -349,14 +366,7 @@ function getTargetFile(aData, aSniffer, aContentType, aIsDocument, aSkipPrompt,
     fp.init(window, bundle.GetStringFromName(titleKey), 
             Components.interfaces.nsIFilePicker.modeSave);
     
-    var urlExt = null;
-    try {
-      var url = aSniffer.uri.QueryInterface(Components.interfaces.nsIURL);
-      urlExt = url.fileExtension;
-    }
-    catch (e) {
-    }
-    appendFiltersForContentType(fp, aContentType, urlExt,
+    appendFiltersForContentType(fp, aContentType, defaultExtension,
                                 aIsDocument ? MODE_COMPLETE : MODE_FILEONLY);  
   
     if (dir)
@@ -374,7 +384,7 @@ function getTargetFile(aData, aSniffer, aContentType, aIsDocument, aSkipPrompt,
     fp.defaultString = defaultString;
   
     if (fp.show() == Components.interfaces.nsIFilePicker.returnCancel || !fp.file)
-      return;
+      return null;
   
     var useDownloadDir = false;
     try {
@@ -411,13 +421,13 @@ function getTargetFile(aData, aSniffer, aContentType, aIsDocument, aSkipPrompt,
     while (file.exists()) {
       var parts = /.+-(\d+)(\..*)?$/.exec(file.leafName);
       if (parts) {
-        file.leafName = file.leafName.replace(/((\d+)\.)/, 
-                                              function (str, p1, part, s) { 
-                                                return (parseInt(part) + 1) + "."; 
+        file.leafName = file.leafName.replace(/((\d+)\.)|((\d+)$)/,
+                                              function (str, dot, dotNum, noDot, noDotNum, pos, s) {
+                                                return (parseInt(str) + 1) + (dot ? "." : "");
                                               });
       }
       else {
-        file.leafName = file.leafName.replace(/\./, "-1$&");
+        file.leafName = file.leafName.replace(/\.|$/, "-1$&");
       }
     }
     
@@ -455,8 +465,7 @@ nsHeaderSniffer.prototype = {
   QueryInterface: function (iid) {
     if (!iid.equals(Components.interfaces.nsIRequestObserver) &&
         !iid.equals(Components.interfaces.nsISupports) &&
-        !iid.equals(Components.interfaces.nsIInterfaceRequestor) &&
-        !iid.equals(Components.interfaces.nsIAuthPrompt)) {
+        !iid.equals(Components.interfaces.nsIInterfaceRequestor)) {
       throw Components.results.NS_ERROR_NO_INTERFACE;
     }
     return this;
@@ -464,37 +473,14 @@ nsHeaderSniffer.prototype = {
 
   // ---------- nsIInterfaceRequestor methods ----------
   getInterface : function(iid) {
-    return this.QueryInterface(iid);
-  },
-
-  // ---------- nsIAuthPrompt methods ----------
-  prompt : function(dlgTitle, text, pwrealm, savePW, defaultText, result)
-  {
-    dump("authprompt prompt! pwrealm="+pwrealm+"\n");
-    var promptServ = this.promptService;
-    if (!promptServ)
-      return false;
-    var saveCheck = {value:savePW};
-    return promptServ.prompt(window, dlgTitle, text, defaultText, pwrealm, saveCheck);
-  },
-  promptUsernameAndPassword : function(dlgTitle, text, pwrealm, savePW, user, pw)
-  {
-    dump("authprompt promptUsernameAndPassword!  "+dlgTitle+" "+text+", pwrealm="+pwrealm+"\n");
-    var promptServ = this.promptService;
-    if (!promptServ)
-      return false;
-    var saveCheck = {value:savePW};
-    return promptServ.promptUsernameAndPassword(window, dlgTitle, text, user, pw, pwrealm, saveCheck);
-  },
-  promptPassword : function(dlgTitle, text, pwrealm, savePW, pw)
-  {
-    dump("auth promptPassword!  "+dlgTitle+" "+text+", pwrealm="+pwrealm+"\n");
-    var promptServ = this.promptService;
-    if (!promptServ)
-      return false;
-
-    var saveCheck = {value:savePW};
-    return promptServ.promptPassword(window, dlgTitle, text, pw, pwrealm, saveCheck);
+    if (iid.equals(Components.interfaces.nsIAuthPrompt)) {
+      // use the window watcher service to get a nsIAuthPrompt impl
+      var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
+                         .getService(Components.interfaces.nsIWindowWatcher);
+      return ww.getNewAuthPrompter(window);
+    }
+    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
+    return null;
   },
 
   // ---------- nsIRequestObserver methods ----------
@@ -517,11 +503,9 @@ nsHeaderSniffer.prototype = {
           // corresponding to each encoding starting from the end, so the first
           // thing it returns corresponds to the outermost encoding.
           var encodingEnumerator = encodedChannel.contentEncodings;
-          if (encodingEnumerator && encodingEnumerator.hasMoreElements()) {
+          if (encodingEnumerator && encodingEnumerator.hasMore()) {
             try {
-              this.contentEncodingType =
-                encodingEnumerator.getNext().
-                  QueryInterface(Components.interfaces.nsISupportsCString).data;
+              this.contentEncodingType = encodingEnumerator.getNext();
             } catch (e) {
             }
           }
@@ -618,16 +602,29 @@ function appendFiltersForContentType(aFilePicker, aContentType, aFileExtension,
       var extEnumerator = mimeInfo.getFileExtensions();
 
       var extString = "";
+      var defaultDesc = "";
+      var plural = false;
       while (extEnumerator.hasMore()) {
+        if (defaultDesc) {
+          defaultDesc += ", ";
+          plural = true;
+        }
         var extension = extEnumerator.getNext();
         if (extString)
           extString += "; ";    // If adding more than one extension,
                                 // separate by semi-colon
         extString += "*." + extension;
+        defaultDesc += extension.toUpperCase();
       }
 
       if (extString) {
-        aFilePicker.appendFilter(mimeInfo.Description, extString);
+        var desc = mimeInfo.Description;
+        if (!desc) { 
+          var key = plural ? "unknownDescriptionFilesPluralFilter" : 
+                             "unknownDescriptionFilesFilter";
+          desc = getStringBundle().formatStringFromName(key, [defaultDesc], 1);
+        }
+        aFilePicker.appendFilter(desc, extString);
       } else {
         aFilePicker.appendFilters(Components.interfaces.nsIFilePicker.filterAll);
       }        
@@ -677,9 +674,15 @@ function makeWebBrowserPersist()
 function makeURL(aURL)
 {
   var ioService = Components.classes["@mozilla.org/network/io-service;1"]
-                .getService(Components.interfaces.nsIIOService);
+                            .getService(Components.interfaces.nsIIOService);
   return ioService.newURI(aURL, null, null);
-  
+}
+
+function makeFileURL(aFile)
+{
+  var ioService = Components.classes["@mozilla.org/network/io-service;1"]
+                .getService(Components.interfaces.nsIIOService);
+  return ioService.newFileURI(aFile);
 }
 
 function makeFilePicker()
@@ -756,20 +759,27 @@ function getDefaultFileName(aDefaultFileName, aNameFromHeaders, aDocumentURI, aD
     // 4) Use the caller-provided name, if any
     return validateFileName(aDefaultFileName);
 
+  // 5) If this is a directory, use the last directory name
+  var re = /\/([^\/]+)\/$/;
+  var path = aDocumentURI.path.match(re);
+  if (path && path.length > 1) {
+      return validateFileName(path[1]);
+  }
+  
   try {
     if (aDocumentURI.host)
-      // 5) Use the host.
+      // 6) Use the host.
       return aDocumentURI.host;
   } catch (e) {
     // Some files have no information at all, like Javascript generated pages
   }
   try {
-    // 6) Use the default file name
+    // 7) Use the default file name
     return getStringBundle().GetStringFromName("DefaultSaveFileName");
   } catch (e) {
     //in case localized string cannot be found
   }
-  // 7) If all else fails, use "index"
+  // 8) If all else fails, use "index"
   return "index";
 }
 
@@ -877,4 +887,11 @@ function getCharsetforSave(aDocument)
   return false;
 }
 
+function SwitchTextEntryDirection(aElement) {
+  if (window.getComputedStyle(aElement, "").direction == "ltr")
+    aElement.style.direction = "rtl";
+  else
+    aElement.style.direction = "ltr";
+}
+
 
